/*
 * cmduqueue.c - cmdu queue utility functions.
 *
 * Copyright (C) 2021 IOPSYS Software Solutions AB. All rights reserved.
 *
 * Author: anjan.chanda@iopsys.eu
 *
 * See LICENSE file for license related information.
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>

#include "timer.h"
#include "bufutil.h"
#include "cmdu.h"

#include "cmduqueue.h"

int cmdu_queue_init(struct cmdu_queue *q)
{
	INIT_LIST_HEAD(&q->q);
	pthread_mutex_init(&q->lock, NULL);
	pthread_cond_init(&q->empty, NULL);

	return 0;
}

int cmdu_queue_destroy(struct cmdu_queue *q)
{
	pthread_mutex_destroy(&q->lock);
	pthread_cond_destroy(&q->empty);

	return 0;
}

void cmdu_queue_flush(struct cmdu_queue *q)
{
	pthread_mutex_lock(&q->lock);
	//list_flush(&q->q, struct cmdu_buffer, list);
	//TODO: free cmdu_buff
	pthread_mutex_unlock(&q->lock);
}

#if 0
static int __cmdu_queue_length(struct cmdu_queue *q)
{
	struct list_head *p;
	int count = 0;

	list_for_each(p, &q->q)
		count++;

	return count;
}
#endif

void cmdu_queue_wait(struct cmdu_queue *q)
{
	pthread_mutex_lock(&q->lock);
	while (list_empty(&q->q))
		pthread_cond_wait(&q->empty, &q->lock);

	pthread_mutex_unlock(&q->lock);
}

void cmdu_queue_signal(struct cmdu_queue *q)
{
	pthread_mutex_lock(&q->lock);
	if (!list_empty(&q->q))
		pthread_cond_signal(&q->empty);

	pthread_mutex_unlock(&q->lock);
}

void cmdu_enqueue(struct cmdu_queue *q, struct cmdu_buff *m)
{
	pthread_mutex_lock(&q->lock);
	list_add_tail(&m->list, &q->q);
	/* fprintf(stderr, "After enqueue msg (%p),  qlen =%d\n",
		m, __cmdu_queue_length(q)); */
	pthread_mutex_unlock(&q->lock);
}

static struct cmdu_buff *__cmdu_dequeue(struct cmdu_queue *q)
{
	struct cmdu_buff *m = NULL;

	m = list_first_entry(&q->q, struct cmdu_buff, list);
	if (m)
		list_del(&m->list);

	return m;
}

struct cmdu_buff *cmdu_dequeue(struct cmdu_queue *q)
{
	struct cmdu_buff *m;

	pthread_mutex_lock(&q->lock);
	while (list_empty(&q->q))
		pthread_cond_wait(&q->empty, &q->lock);

	m = __cmdu_dequeue(q);
	/* fprintf(stderr, "After dequeue msg (%p),  qlen =%d\n",
		m, __cmdu_queue_length(q)); */

	pthread_mutex_unlock(&q->lock);

	return m;
}
