/*
 * worker.c - worker thread functions
 *
 * Copyright (C) 2021 IOPSYS Software Solutions AB. All rights reserved.
 *
 * Author: anjan.chanda@iopsys.eu
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>

#include <libubox/list.h>

#include "worker.h"

static void *worker_run(void *data)
{
	struct worker *w = (struct worker *)data;

	for (;;)
		w->func(w->data);

	return NULL;
}

int worker_init(struct worker *w)
{
	struct sched_param sp;
	int ret;

	ret = pthread_attr_init(&w->attr);
	if (!ret) {
		sp.sched_priority = 99;
		pthread_attr_setinheritsched(&w->attr, PTHREAD_EXPLICIT_SCHED);
		pthread_attr_setschedpolicy(&w->attr, SCHED_RR);
		pthread_attr_setschedparam(&w->attr, &sp);
	}

	ret = pthread_create(&w->id, NULL, worker_run, w);
	if (ret)
		return -1;

	return 0;
}

void worker_exit(struct worker *w)
{
	pthread_cancel(w->id);
	pthread_join(w->id, NULL);
}
